@@ -40,7 +40,6 @@
 #include "gnome-rr.h"
 #include "gnome-rr-config.h"
 
-#include "private.h"
 #include "edid.h"
 #include "gnome-rr-private.h"
 
@@ -69,6 +68,7 @@ struct GnomeRROutput
     RROutput		id;
     
     char *		name;
+    char *		display_name;
     GnomeRRCrtc *	current_crtc;
     gboolean		connected;
     gulong		width_mm;
@@ -835,6 +835,9 @@ gnome_rr_screen_class_init (GnomeRRScreenClass *klass)
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
     g_type_class_add_private (klass, sizeof (GnomeRRScreenPrivate));
 
+    bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
     gobject_class->set_property = gnome_rr_screen_set_property;
     gobject_class->get_property = gnome_rr_screen_get_property;
     gobject_class->finalize = gnome_rr_screen_finalize;
@@ -969,8 +972,6 @@ gnome_rr_screen_new (GdkScreen *screen,
     if (rr_screen)
 	g_object_ref (rr_screen);
     else {
-	_gnome_desktop_init_i18n ();
-
 	rr_screen = g_initable_new (GNOME_TYPE_RR_SCREEN, NULL, error, "gdk-screen", screen, NULL);
 	if (rr_screen) {
 	    g_object_set_data (G_OBJECT (screen), "GnomeRRScreen", rr_screen);
@@ -1582,6 +1583,7 @@ output_initialize (GnomeRROutput *output, XRRScreenResources *res, GError **erro
     }
     
     output->name = g_strdup (info->name); /* FIXME: what is nameLen used for? */
+    output->display_name = NULL; /* set first time the getter is used */
     output->current_crtc = crtc_by_id (output->info, info->crtc);
     output->width_mm = info->mm_width;
     output->height_mm = info->mm_height;
@@ -1695,6 +1697,7 @@ output_free (GnomeRROutput *output)
     g_free (output->possible_crtcs);
     g_free (output->edid_data);
     g_free (output->name);
+    g_free (output->display_name);
     g_free (output->connector_type);
     g_slice_free (GnomeRROutput, output);
 }
@@ -1716,6 +1719,13 @@ gnome_rr_output_get_edid_data (GnomeRROutput *output, gsize *size)
     return output->edid_data;
 }
 
+/**
+ * gnome_rr_output_get_ids_from_edid:
+ * @output: a #GnomeRROutput
+ * @vendor: (out) (allow-none):
+ * @product: (out) (allow-none):
+ * @serial: (out) (allow-none):
+ */
 gboolean
 gnome_rr_output_get_ids_from_edid (GnomeRROutput         *output,
                                    char                 **vendor,
@@ -1744,6 +1754,52 @@ gnome_rr_output_get_ids_from_edid (GnomeRROutput         *output,
 
 }
 
+static void
+ensure_display_name (GnomeRROutput *output)
+{
+    if (output->display_name != NULL)
+        return;
+
+    if (gnome_rr_output_is_laptop (output))
+        output->display_name = g_strdup (_("Laptop"));
+
+    if (output->display_name == NULL
+        && output->edid_data != NULL) {
+        MonitorInfo *info;
+
+        info = decode_edid (output->edid_data);
+        if (info != NULL)
+            output->display_name = make_display_name (info);
+
+        g_free (info);
+    }
+
+    if (output->display_name == NULL) {
+        char *inches;
+        inches = make_display_size_string (output->width_mm, output->height_mm);
+        if (inches != NULL) {
+            /* Translators: %s is the size of the monitor in inches */
+            output->display_name = g_strdup_printf (_("%s Display"), inches);
+        }
+        g_free (inches);
+    }
+
+    /* last chance on the stairway */
+    if (output->display_name == NULL) {
+      output->display_name = g_strdup (_("Unknown Display"));
+    }
+}
+
+const char *
+gnome_rr_output_get_display_name (GnomeRROutput *output)
+{
+    g_return_val_if_fail (output != NULL, NULL);
+
+    ensure_display_name (output);
+
+    return output->display_name;
+}
+
 /**
  * gnome_rr_output_get_backlight_min:
  *
@@ -1970,6 +2026,12 @@ gnome_rr_output_get_current_mode (GnomeRROutput *output)
     return NULL;
 }
 
+/**
+ * gnome_rr_output_get_position:
+ * @output: a #GnomeRROutput
+ * @x: (out) (allow-none):
+ * @y: (out) (allow-none):
+ */
 void
 gnome_rr_output_get_position (GnomeRROutput   *output,
 			      int             *x,
@@ -2247,6 +2309,12 @@ gnome_rr_crtc_can_drive_output (GnomeRRCrtc   *crtc,
 }
 
 /* FIXME: merge with get_mode()? */
+/**
+ * gnome_rr_crtc_get_position:
+ * @crtc: a #GnomeRRCrtc
+ * @x: (out) (allow-none):
+ * @y: (out) (allow-none):
+ */
 void
 gnome_rr_crtc_get_position (GnomeRRCrtc *crtc,
 			    int         *x,
